﻿<#
 ██████╗██╗██╗   ██╗██╗██╗     ██╗███████╗ █████╗ ████████╗██╗ ██████╗ ███╗   ██╗    ██╗   ██╗██╗██╗
██╔════╝██║██║   ██║██║██║     ██║╚══███╔╝██╔══██╗╚══██╔══╝██║██╔═══██╗████╗  ██║    ██║   ██║██║██║
██║     ██║██║   ██║██║██║     ██║  ███╔╝ ███████║   ██║   ██║██║   ██║██╔██╗ ██║    ██║   ██║██║██║
██║     ██║╚██╗ ██╔╝██║██║     ██║ ███╔╝  ██╔══██║   ██║   ██║██║   ██║██║╚██╗██║    ╚██╗ ██╔╝██║██║
╚██████╗██║ ╚████╔╝ ██║███████╗██║███████╗██║  ██║   ██║   ██║╚██████╔╝██║ ╚████║     ╚████╔╝ ██║██║
 ╚═════╝╚═╝  ╚═══╝  ╚═╝╚══════╝╚═╝╚══════╝╚═╝  ╚═╝   ╚═╝   ╚═╝ ╚═════╝ ╚═╝  ╚═══╝      ╚═══╝  ╚═╝╚═╝ 

SCRIPT INFO:
    Name:      Generate_Summary.ps1
    Author:    Janne Palo
    Company:   virtuaalimedia.fi
    License:   Any (Totally Free)
    Version:   1.1
    Created:   2025-03-01

DESCRIPTION:
    This script searches for files of a specified extension within the script's directory (or a custom path),
    logs all found files (even empty ones) into a summary file, and combines the content of non-empty files into 
    a single output file. If an output file already exists, it is versioned automatically (V1, V2, etc.).

DEFAULT BEHAVIOR:
    - Searches for files in the **same directory as the script**.
    - Saves output files in the **same directory as the script**.
    - Lists all found files in the summary (including empty ones).
    - If the output file already exists, a version number (`V1`, `V2`, etc.) is appended.

PARAMETERS:
    -SkipConfirm   : Skips the confirmation prompt.
    -SearchPath    : Custom directory to search for files (default: script's directory).
    -OutputPath    : Custom directory for output files (default: script's directory).
    -OutputFile    : Custom filename for the combined output (default: auto-generated with versioning).
    -Extension     : File extension to summarize (e.g., .js, .xml, .sql).
    -Exclusions    : Comma-separated list of substrings to exclude from filenames.

EXAMPLES:
    .\Generate_Summary.ps1 -Extension .js -SkipConfirm
    .\Generate_Summary.ps1 -Extension .xml -SearchPath "C:\Logs" -OutputPath "D:\Summaries"
#>

param (
    [switch]$SkipConfirm,
    [string]$SearchPath,
    [string]$OutputPath,
    [string]$OutputFile,
    [string]$Extension,
    [string]$Exclusions
)

# Ensure PowerShell uses UTF-8 encoding
[Console]::OutputEncoding = [System.Text.Encoding]::UTF8

# Get the script's directory (default search & output location)
$scriptDir = Split-Path -Parent $MyInvocation.MyCommand.Definition
$searchDir = if ($SearchPath) { $SearchPath } else { $scriptDir }

# Get file extension (from argument or prompt)
if (-not $Extension) {
    $Extension = Read-Host "Which file extension would you like to summarize? (Example: .js, .xml, .sql)"
}
if (-not $Extension) {
    Write-Host "No extension chosen. Exiting."
    exit
}
if (-not $Extension.StartsWith(".")) {
    $Extension = ".$Extension"
}

# Get exclusions (from argument or prompt)
if (-not $Exclusions) {
    $Exclusions = Read-Host "Enter a comma-separated list of filename substrings to exclude (or leave blank if none):"
}
$ExclusionList = $Exclusions -split ',' | ForEach-Object { $_.Trim() } | Where-Object { $_ -ne '' }

# Build filenames (always using .txt for output files)
$extWithoutDot = $Extension.Replace('.', '')
$extPrefix = $extWithoutDot.ToUpper()
$today = Get-Date -Format "dd_MM_yyyy"

# Set output directory
$outputDir = if ($OutputPath) { $OutputPath } else { $scriptDir }

# Ensure output filename (with versioning)
$baseOutputFile = if ($OutputFile) { $OutputFile } else { "${extPrefix}_all_files_${today}.txt" }
$version = 1
$combinedFile = Join-Path $outputDir $baseOutputFile
while (Test-Path $combinedFile) {
    $combinedFile = Join-Path $outputDir "${extPrefix}_all_files_${today}_V$version.txt"
    $version++
}

# Summary file (always named consistently)
$summaryFile = Join-Path $outputDir "${extPrefix}_FilesSummary.txt"

# Show confirmation (unless skipped)
if (-not $SkipConfirm) {
    Write-Host "`n⚠️ WARNING: You are about to process **ALL $Extension files** in: $searchDir" -ForegroundColor Red
    Write-Host "📂 Output files will be saved in: $outputDir" -ForegroundColor Yellow
    Write-Host "📜 Summary file: $summaryFile"
    Write-Host "📝 Combined output file: $combinedFile"
    $confirm = Read-Host "Proceed? (y/N)"
    if ($confirm -ne "y") {
        Write-Host "🚫 Operation canceled."
        exit
    }
}

# Overwrite or create an empty summary file
[System.IO.File]::WriteAllText($summaryFile, "📝 Summary of files with extension '$Extension' on $(Get-Date):`r`n", [System.Text.Encoding]::UTF8)

# 🔍 Searching for files
Write-Host "`n🔍 Searching for files..."
$allFiles = Get-ChildItem -Path $searchDir -Filter "*$Extension" -Recurse -File

# Apply exclusions
$files = $allFiles | Where-Object {
    $filePath = $_.FullName
    -not ($ExclusionList | Where-Object { $filePath -like "*$_*" })
}

Write-Host "✅ Found $($files.Count) files after exclusions.`n"

foreach ($file in $files) {
    $filePath = (Resolve-Path $file.FullName).ProviderPath

    # Log file to summary (even empty ones)
    Add-Content -Path $summaryFile -Value "📄 $filePath (Size: $(Get-Item $filePath).Length bytes)" -Encoding UTF8
}

# Open the combined file once in append/write mode
$writeStream = [System.IO.File]::Open($combinedFile, 'Append', 'Write', 'Read')

# 🏗 Processing files
Write-Host "🛠 Processing files..."

try {
    foreach ($file in $files) {
        $filePath = (Resolve-Path $file.FullName).ProviderPath

        # Skip if it's empty
        if ((Get-Item $filePath).Length -eq 0) {
            continue
        }

        # Write a small header for clarity in the combined file
        $marker = [System.Text.Encoding]::UTF8.GetBytes("`r`n/*--- 📄 File: $filePath ---*/`r`n")
        $writeStream.Write($marker, 0, $marker.Length)

        # Open the source file in read mode
        $readStream = [System.IO.File]::OpenRead($filePath)

        # Copy data in chunks to the combined file
        $buffer = New-Object byte[] 4096
        while ($true) {
            $bytesRead = $readStream.Read($buffer, 0, $buffer.Length)
            if ($bytesRead -le 0) { break }
            $writeStream.Write($buffer, 0, $bytesRead)
        }
        $readStream.Close()
    }
}
finally {
    $writeStream.Close()
}

Write-Host "`b✅ Done!"
Write-Host "✅ Appended all matching files into: $combinedFile"
Write-Host "📜 File details logged in: $summaryFile"
