"""
 ██████╗██╗██╗   ██╗██╗██╗     ██╗███████╗ █████╗ ████████╗██╗ ██████╗ ███╗   ██╗    ██╗   ██╗██╗██╗
██╔════╝██║██║   ██║██║██║     ██║╚══███╔╝██╔══██╗╚══██╔══╝██║██╔═══██╗████╗  ██║    ██║   ██║██║██║
██║     ██║██║   ██║██║██║     ██║  ███╔╝ ███████║   ██║   ██║██║   ██║██╔██╗ ██║    ██║   ██║██║██║
██║     ██║╚██╗ ██╔╝██║██║     ██║ ███╔╝  ██╔══██║   ██║   ██║██║   ██║██║╚██╗██║    ╚██╗ ██╔╝██║██║
╚██████╗██║ ╚████╔╝ ██║███████╗██║███████╗██║  ██║   ██║   ██║╚██████╔╝██║ ╚████║     ╚████╔╝ ██║██║
 ╚═════╝╚═╝  ╚═══╝  ╚═╝╚══════╝╚═╝╚══════╝╚═╝  ╚═╝   ╚═╝   ╚═╝ ╚═════╝ ╚═╝  ╚═══╝      ╚═══╝  ╚═╝╚═╝                                                                                                 
    
SCRIPT INFO:
    Name:      Generate_Summary.py
    Author:    Janne Palo
    Company:   virtuaalimedia.fi
    License:   Any (Totally Free)
    Version:   1.0
    Created:   2025-03-01

📜 File Summarization Script 📜
--------------------------------
This script searches for all files of a specified extension within the script's directory (and its subdirectories),
logs all found files (even empty ones) into a summary file, and combines the content of non-empty files into a single
output file. By default, both the summary and combined files are stored in the same directory as the script.

Command-line options allow customization of the search path, output directory, output filename, file extension, and 
exclusions. If no arguments are provided, the script runs in interactive mode.

💡 Default behavior:
   - Searches for the specified file type in the **same directory as this script**.
   - Saves output files in the **same directory as this script**.
   - If the output file already exists, a version number (`V1`, `V2`, etc.) is appended.
"""

import os
import sys
import time
import datetime
import glob
import shutil
import argparse

# 🛠️ Argument Parsing (for optional command-line arguments)
parser = argparse.ArgumentParser(
    description="Summarizes and combines files of a specified extension. By default, the script searches for files in its own directory (and subdirectories) and stores output files there."
)

parser.add_argument("--skip-confirm", action="store_true", help="Skip confirmation prompt.")
parser.add_argument("--search-path", type=str, help="Custom directory to search for files (default: script's directory).")
parser.add_argument("--output-path", type=str, help="Custom directory for output files (default: script's directory).")
parser.add_argument("--output-file", type=str, help="Custom filename for the combined output (default: auto-generated).")
parser.add_argument("--extension", type=str, help="File extension to summarize (e.g., .js, .xml, .sql).")
parser.add_argument("--exclusions", type=str, help="Comma-separated list of substrings to exclude from filenames.")

args = parser.parse_args()

# Get the script's directory (default search & output location)
script_dir = os.path.dirname(os.path.abspath(__file__))

# Set search directory
search_dir = args.search_path if args.search_path else script_dir

# Get file extension (from argument or prompt)
if args.extension:
    extension = args.extension.strip()
else:
    extension = input("Which file extension would you like to summarize? (Example: .js, .xml, .sql): ").strip()

if not extension:
    print("No extension chosen. Exiting.")
    sys.exit()

# Ensure the extension starts with a dot
if not extension.startswith("."):
    extension = "." + extension

# Get exclusions (from argument or prompt)
if args.exclusions:
    exclusions = [e.strip() for e in args.exclusions.split(",") if e.strip()]
else:
    exclusions_input = input("Enter a comma-separated list of filename substrings to exclude (or leave blank if none): ").strip()
    exclusions = [e.strip() for e in exclusions_input.split(",") if e.strip()]

# Build filenames (always using .txt for output files)
ext_without_dot = extension.lstrip(".")
ext_prefix = ext_without_dot.upper()
today = datetime.datetime.now().strftime("%d_%m_%Y")

# Set output directory
output_dir = args.output_path if args.output_path else script_dir

# Ensure output filename (with versioning)
base_output_file = args.output_file if args.output_file else f"{ext_prefix}_all_files_{today}.txt"

# Auto-versioning for output file
version = 1
output_file = os.path.join(output_dir, base_output_file)
while os.path.exists(output_file):
    output_file = os.path.join(output_dir, f"{ext_prefix}_all_files_{today}_V{version}.txt")
    version += 1

# Summary file (always named consistently)
summary_file = os.path.join(output_dir, f"{ext_prefix}_FilesSummary.txt")

# Show confirmation (unless skipped)
if not args.skip_confirm:
    print(f"\n⚠️ WARNING: You are about to process **ALL {extension} files** in: {search_dir}")
    print(f"📂 Output files will be saved in: {output_dir}")
    print(f"📜 Summary file: {summary_file}")
    print(f"📝 Combined output file: {output_file}")
    confirm = input("Proceed? (y/N): ").strip().lower()
    if confirm != "y":
        print("🚫 Operation canceled.")
        sys.exit()

# Overwrite or create empty summary file
with open(summary_file, "w", encoding="utf-8") as sf:
    sf.write(f"📝 Summary of files with extension '{extension}' on {datetime.datetime.now()}:\n")

# 🌀 Optimized File Search
spinner = ["|", "/", "-", "\\"]
spinner_index = 0
print("\n🔍 Searching for files ", end="", flush=True)

matching_files = []
for file_path in glob.iglob(f"{search_dir}/**/*{extension}", recursive=True):
    file_name = os.path.basename(file_path)

    # Log file to summary (even empty ones)
    with open(summary_file, "a", encoding="utf-8") as sf:
        sf.write(f"📄 {file_path} (Size: {os.path.getsize(file_path)} bytes)\n")

    # Skip files matching exclusion list
    if any(excl in file_name for excl in exclusions):
        continue

    matching_files.append(file_path)

    # Rotate spinner every 200 iterations
    if len(matching_files) % 200 == 0:
        print(f"\b{spinner[spinner_index]}", end="", flush=True)
        spinner_index = (spinner_index + 1) % len(spinner)

print(f"\b✅ Found {len(matching_files)} files.\n")

# 🏗 Optimized Processing with Proper File Closure
spinner_index = 0
print("🛠 Processing files ", end="", flush=True)

try:
    with open(output_file, "a", encoding="utf-8") as cf:
        for i, file_path in enumerate(matching_files):
            # Write a small header for clarity in the combined file
            cf.write(f"\n/*--- 📄 File: {file_path} ---*/\n")

            # Copy file contents only if it's not empty
            if os.path.getsize(file_path) > 0:
                with open(file_path, "r", encoding="utf-8", errors="ignore") as f:
                    shutil.copyfileobj(f, cf)

            # Rotate spinner every 200 files
            if i % 200 == 0:
                print(f"\b{spinner[spinner_index]}", end="", flush=True)
                spinner_index = (spinner_index + 1) % len(spinner)

except Exception as e:
    print(f"\n❌ Error: {e}")
    sys.exit(1)

print(f"\b✅ Done!\n")
print(f"✅ Appended all matching files into: {output_file}")
print(f"📜 File details logged in: {summary_file}")

# Ensure the script exits properly
sys.exit(0)
